import json


# get_markers requests all marker data or valid markers, converts the data to json, and writes
# the data to the response object
def get_all_markers(self, cursor, valid: bool, waterBodyName):
    try:
        # NB: interval temporarily hard coded to 5 days
        if valid:
            cursor.execute('''
                SELECT m.MeasurementID, m.SensorID, m.TimeMeasured, 
                       s.SensorType, s.Active, 
                       b.Name,
                       d.SubDivisionID, d.GroupID, d.MinimumThickness, 
                       d.AverageThickness, d.CenterLatitude, d.CenterLongitude, 
                       d.Accuracy
                FROM Measurement m
                INNER JOIN Sensor s ON m.SensorID = s.SensorID
                INNER JOIN BodyOfWater b ON m.WaterBodyName = b.Name
                LEFT JOIN SubDivision d ON m.MeasurementID = d.MeasurementID
                WHERE m.TimeMeasured > DATE_SUB(NOW(), INTERVAL 5 DAY) & b.Name = ?
            ''', waterBodyName)
        else:
            cursor.execute('''
                SELECT m.MeasurementID, m.SensorID, m.TimeMeasured, 
                       s.SensorType, s.Active, 
                       b.Name,
                       d.SubDivisionID, d.GroupID, d.MinimumThickness, 
                       d.AverageThickness, d.CenterLatitude, d.CenterLongitude, 
                       d.Accuracy
                FROM Measurement m
                INNER JOIN Sensor s ON m.SensorID = s.SensorID
                INNER JOIN BodyOfWater b ON m.WaterBodyName = b.Name
                LEFT JOIN SubDivision d ON m.MeasurementID = d.MeasurementID
                WHERE b.Name = ?
            ''', waterBodyName)

        rows = cursor.fetchall()

        # Container for all fetched measurement objects
        measurement_data = {}

        # Iterate over all fetched rows
        for row in rows:
            measurement_id = row[0]

            # Create subdivision object for current row
            sub_division = {
                'SubDivisionID': row[3],
                'GroupID': row[4],
                'MinimumThickness': row[5],
                'AverageThickness': row[6],
                'CenterLatitude': row[7],
                'CenterlONGITUDE': row[8],
                'Accuracy': row[9]
            }

            # Check if measurement ID already exists in measurement_data
            if measurement_id in measurement_data:
                # Check if the data object already exists in the list
                if sub_division not in measurement_data[measurement_id]['SubDivision']:
                    measurement_data[measurement_id]['SubDivision'].append(sub_division)

            else:
                # Create a new entry for measurement_id if it does not already exist in the list
                measurement_data[measurement_id] = {
                    'MeasurementID': measurement_id,
                    'TimeMeasured': row[2],
                    'Sensor': {  # Each measurement only has one related sensor
                        'SensorID': row[1],
                        'SensorType': row[9],
                        'Active': bool(row[10])
                    },
                    'SubDivision': [sub_division],  # Array of sub_division objects
                    'BodyOfWater': row[14]
                }

        # Convert dictionary values to list of measurements
        data = list(measurement_data.values())

        if len(rows) == 0 or len(data) == 0:  # Return 500 and empty list if no data is found
            print(f"An error occurred while querying the database")
            resp_code = 500
            marker_data = '[]'
        else:
            resp_code = 200
            # Convert list of dictionaries to JSON
            marker_data = json.dumps(data, indent=4)

    except Exception as e:
        print(f"An error occurred while querying the database: {e}")
        resp_code = 500
        marker_data = '[]'

    # Set headers
    self.send_response(resp_code)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Write marker data to response object
    self.wfile.write(marker_data.encode('utf-8'))
